# Data analysis

# Required packages
library(tidyverse)
library(readxl)
library(car)

# Importing data
CDD <- read_excel("Data/cdDispersalData.xlsx", sheet = "CDD")
CDD.SF <- read_excel("Data/cdDispersalData.xlsx", sheet = "CDDSF") # The final counts cast in terms of successes and failures

# Extracting key results from data ####
# The mean number of individuals in patch 1 when the taps were opened
day0.totalAnimals <- CDD %>% 
  filter(day == 0 & patch == 1) %>% 
  summarise(mean.pop.size=mean(totalAnimals)) %>% 
  print()

# The mean number of individuals in patch 1 when dispersal began by group
day0.totalAnimalsGroup <- CDD %>% 
  filter(day == 0 & patch == 1) %>%
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`")) %>% 
  summarise(mean.pop.size=mean(totalAnimals)) %>% 
  print()

# The mean number of individuals per group at final count
day4.totalAnimalsGroup <- CDD %>%
  filter(day == 4) %>% 
  group_by(`patch1Fed?`,`patch2Fed?`) %>% 
  summarise(totalAnimals=mean(totalAnimals[patch == "2"] + (totalAnimals[patch == "1"]))) %>% 
  print()

# The number of individuals per group per replicate at final count
day4.totalAnimalsGroupReplicate <- CDD %>%
  filter(day == 4) %>% 
  group_by(`patch1Fed?`,`patch2Fed?`,`replicate`) %>% 
  summarise(totalAnimals=totalAnimals[patch == "2"] + (totalAnimals[patch == "1"])) %>% 
  print()

# The number of individuals in patch 2 per group per replicate at final count
day4.dispersedReplicate <- CDD %>%
  filter(day == 4 & patch == 2) %>% 
  group_by(`patch1Fed?`,`patch2Fed?`,`replicate`) %>% 
  summarise(totalAnimals) %>% 
  print()

# The mean number of individuals per patch per group at final count
day4.meanAbsDispersed <- CDD %>%
  filter(day == 4) %>% 
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","patch")) %>% 
  summarise(totalAnimals=mean(totalAnimals),adults=mean(adults),juveniles=mean(juveniles)) %>% 
  print()

# The mean proportion of juveniles per patch per group at final count
day4.meanPropJuv <- CDD %>%
  filter(day == 4) %>% 
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","patch")) %>% 
  summarise(propJuvs=(mean(juveniles)/mean(totalAnimals))) %>% 
  print()

# The mean proportion of juveniles per patch at final count
day4.meanPropJuvNoGroup <- CDD %>%
  filter(day == 4) %>% 
  group_by(.dots=c("patch")) %>% 
  summarise(propJuvs=(mean(juveniles)/mean(totalAnimals))) %>% 
  print()

# The mean number of individuals in patch 2 per group at final count
day4.meanDispersersPatch2 <- CDD %>%
  filter(day == 4 & patch == 2) %>% 
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`")) %>% 
  summarise(totalAnimals=mean(totalAnimals)) %>% 
  print()

# The mean number of individuals in patch 2 based on patch 1 at final count
day4.meanDispersersPatch2BasedOn1 <- CDD %>%
  filter(day == 4 & patch == 2) %>% 
  group_by(.dots=c("`patch1Fed?`")) %>% 
  summarise(meanTotalAnimals=mean(totalAnimals),
            n = n(),
            se = sd(totalAnimals)/sqrt(n)) %>% 
  print()

# The mean proportion of dispersers per replicate at final count, with SE
day4.meanPropDispersed <- CDD %>%
    filter(day == 4) %>% 
    group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","replicate","patch")) %>% 
    summarise(totalAnimals=mean(totalAnimals)) %>% 
    summarise(propDispersed=totalAnimals[patch == "2"] / (totalAnimals[patch == "1"] + totalAnimals[patch == "2"]),
              n=totalAnimals[patch=="2"]+totalAnimals[patch=="1"]) %>% 
    mutate(SE = sqrt((propDispersed*(1-propDispersed))/n)) %>% 
    print()

# The mean proportion of dispersers per group at final count
day4.meanPropDispersedGroup <- day4.meanPropDispersed %>%
    group_by(.dots=c("`patch1Fed?`","`patch2Fed?`")) %>% 
    summarise(propDispersed=mean(propDispersed)) %>% 
    print()

# The mean proportion of dispersers depending on whether patch 1 was fed or not at final count, with SE
day4.meanPropDispersedPatch1 <- day4.meanPropDispersed %>%
    group_by(`patch1Fed?`) %>% 
    summarise(meanPropDispersed=mean(propDispersed),
              n = n(),
              standard.dev = sd(propDispersed)) %>% 
    mutate(SE = standard.dev / sqrt(n),
           lower.ci = meanPropDispersed - qt(1 - (0.05 / 2), n - 1) * SE,
           upper.ci = meanPropDispersed + qt(1 - (0.05 / 2), n - 1) * SE) %>% 
    print()

# Statistical analysis ####
# The proportion of dispersers after 96 hours
# Sumamrised data
day4.meanPropDispersedGroup

# Performing test
CDD.SF.glm <- glm(as.matrix(CDD.SF[,c("successes","failures")]) ~ `patch1Fed?`*`patch2Fed?`, data = CDD.SF, family = binomial)
summary(CDD.SF.glm) # Significant effect of food availability in patch 1

# The total population size of each system after 96 hours
# Summarised data
day4.totalAnimalsGroup

# Checking assumptions and performing test (ANOVA)
boxplot(day4.totalAnimalsGroupReplicate$totalAnimals~day4.totalAnimalsGroupReplicate$`patch1Fed?`*day4.totalAnimalsGroupReplicate$`patch2Fed?`)
finalPopSize.aov <- aov(totalAnimals ~ `patch1Fed?`*`patch2Fed?`, data = day4.totalAnimalsGroupReplicate)
plot(finalPopSize.aov)
finalPopSize.residuals <- residuals(object = finalPopSize.aov)
shapiro.test(x = finalPopSize.residuals) # Shapiro-Wilk test of normality
# W = 0.92659 and p-value = 0.1327 means that we cannot reject the hypothesis that the sample comes from a population which has a normal distribution
# Normality check passed
leveneTest(finalPopSize.aov) # Levene test for homogeneity of variances
# p-value = 0.4583 means that we cannot reject the hypothesis that the samples come from groups with equal variances
# Homogeneity of variances check passed
# Assumptions not violated
summary(finalPopSize.aov)

# The absoluate number of dispersers after 96 hours
# Summarised data, ignoring `patch2Fed?`
day4.meanDispersersPatch2

# Checking assumptions and performing test (ANOVA)
boxplot(day4.dispersedReplicate$totalAnimals~day4.dispersedReplicate$`patch1Fed?`*day4.dispersedReplicate$`patch2Fed?`)
finalAbsDispersers.aov <- aov(totalAnimals ~ `patch1Fed?`*`patch2Fed?`, data = day4.dispersedReplicate)
plot(finalAbsDispersers.aov)
# Assumption tests
finalAbsDispersers.residuals <- residuals(object = finalAbsDispersers.aov)
shapiro.test(x = finalAbsDispersers.residuals) # Shapiro-Wilk test of normality
# W = 0.93031 and p-value = 0.1566 means that we cannot reject the hypothesis that the sample comes from a population which has a normal distribution
# Normality check passed
leveneTest(finalAbsDispersers.aov) # Levene test for homogeneity of variances
# p-value = 0.2242 means that we cannot reject the hypothesis that the samples come from groups with equal variances
# Homogeneity of variances check passed
# Assumptions not violated
summary(finalAbsDispersers.aov)

# The of effect of patch 1 fed / not fed on the proportion of disperspers by population density
# Summarised data
day4.meanPropDispersed

# Checking assumptions and performing test
boxplot(propDispersed~`patch1Fed?`, data = day4.meanPropDispersed)
CDD.SF$n <- CDD.SF$successes+CDD.SF$failures
dens.glm <- glm(as.matrix(CDD.SF[,c("successes","failures")]) ~ `patch1Fed?`*n, data = CDD.SF, family = binomial(link = "logit"))
summary(dens.glm) # Residual deviance is markedly higher than degrees of freedom, indicating a likely violation of equal variances. Switching to quasi-binomial distribution
densQ.glm <- glm(as.matrix(CDD.SF[,c("successes","failures")]) ~ `patch1Fed?`*n, data = CDD.SF, family = quasibinomial(link = "logit"))
summary(densQ.glm) # Significant effect of food availability in patch 1, not of interaction nor density itself

# The effect of patch on the proportion of juveniles
# Summarised data
day4.meanPropJuvNoGroup

# Manipulating data, checking assumptions, and performing test
juvDisp <- CDD %>% 
  filter(day == 4) %>% 
  group_by(.dots=c("patch"))

juvDisp.glm <- glm(as.matrix(juvDisp[,c("juveniles","totalAnimals")]) ~ patch, data = juvDisp, family = binomial)
summary(juvDisp.glm) # No significant effect of patch on proportion of juveniles

# Figures ####
# The effect of treatment on the proportion of the population that dispersed
# Manipulating data for plotting
day4.meanPropDispersed$patch1Fed.patch2Fed <- interaction(day4.meanPropDispersed$`patch1Fed?`,day4.meanPropDispersed$`patch2Fed?`)
day4.meanPropDispersed$patch1Fed.patch2Fed <- factor(day4.meanPropDispersed$patch1Fed.patch2Fed, levels = c("N.Y","N.N","Y.Y","Y.N"))

# Plotting
ggplot(day4.meanPropDispersed, aes(x=factor(patch1Fed.patch2Fed), y=propDispersed,group=replicate,ymax=propDispersed+SE,ymin=propDispersed-SE)) +
  geom_pointrange(position = position_dodge(width=0.3)) +
  scale_x_discrete(labels = c("Patch 1 not fed \u00D7\n Patch 2 well fed", "Patch 1 not fed \u00D7\n Patch 2 not fed", "Patch 1 well fed \u00D7\n Patch 2 well fed", "Patch 1 well fed \u00D7\n Patch 2 not fed")) +
  scale_y_continuous(expand = c(0, 0), limits = c(-0.01,0.6)) +
  labs(x = " ", y = c(expression("Proportion of"~italic("D. carinata")~"dispersing into patch 2"))) +
  annotate("segment", x=c(1,1,2),xend=c(1,2,2), y= c(0.52,0.53,0.53), yend=c(0.53,0.53,0.52)) +
  annotate("text", x=1.5, y=0.54, label="NS", size=5) +
  annotate("segment", x=c(3,3,4),xend=c(3,4,4), y= c(0.1,0.11,0.11), yend=c(0.11,0.11,0.1)) +
  annotate("text", x=3.5, y=0.12, label="NS", size=5) +
  annotate("segment", x=c(1.5,1.5,3.5),xend=c(1.5,3.5,3.5), y= c(0.56,0.57,0.57), yend=c(0.57,0.57,0.56)) +
  annotate("text", x=2.5, y=0.575, label="***", size=6) +
  theme_classic(base_size = 16) +
  theme(legend.text.align = 0, legend.position = c(0.8,0.5),axis.text.x = element_text(colour = "black"))
#ggsave(filename = "Figures/figTreatmentDispersal.pdf", width = 20, height = 20, units = "cm")

# Plotting with custom x-axis superimposed
ggplot(day4.meanPropDispersed, aes(x=factor(patch1Fed.patch2Fed), y=propDispersed,group=replicate,ymax=propDispersed+SE,ymin=propDispersed-SE)) +
  geom_pointrange(position = position_dodge(width=0.3)) +
  scale_x_discrete(labels = c("Patch 1 not fed \u00D7\n Patch 2 well fed", "Patch 1 not fed \u00D7\n Patch 2 not fed", "Patch 1 well fed \u00D7\n Patch 2 well fed", "Patch 1 well fed \u00D7\n Patch 2 not fed")) +
  scale_y_continuous(expand = c(0, 0), limits = c(-0.01,0.6)) +
  labs(x = " ", y = c(expression("Proportion of"~italic("D. carinata")~"dispersing into patch 2"))) +
  annotate("segment", x=c(1,1,2),xend=c(1,2,2), y= c(0.52,0.53,0.53), yend=c(0.53,0.53,0.52)) +
  annotate("text", x=1.5, y=0.54, label="NS", size=5) +
  annotate("segment", x=c(3,3,4),xend=c(3,4,4), y= c(0.1,0.11,0.11), yend=c(0.11,0.11,0.1)) +
  annotate("text", x=3.5, y=0.12, label="NS", size=5) +
  annotate("segment", x=c(1.5,1.5,3.5),xend=c(1.5,3.5,3.5), y= c(0.56,0.57,0.57), yend=c(0.57,0.57,0.56)) +
  annotate("text", x=2.5, y=0.575, label="***", size=6) +
  theme_classic(base_size = 16) +
  theme(legend.text.align = 0, legend.position = c(0.8,0.5),axis.text.x = element_blank(), 
        plot.margin = unit(c(1,1,2,1),"cm"))
#ggsave(filename = "Figures/figTreatmentDispersalNoX.png", width = 20, height = 20, units = "cm")

# The number of individuals in patch 1 and patch 2 over time
# Manipulating data for plotting
CDD$patch1Fed.patch2Fed <- interaction(CDD$`patch1Fed?`,CDD$`patch2Fed?`)
CDD$patch1Fed.patch2Fed <- factor(CDD$patch1Fed.patch2Fed, levels = c("N.Y","N.N","Y.Y","Y.N"))
allDays.meanAbsDispersed <- CDD %>%
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","patch","day")) %>% 
  summarize(totalPop = mean(totalAnimals), n = n(), 
            sd = sd(totalAnimals), se = sd/sqrt(n),
            ymin = totalPop-se, ymax = totalPop+se) %>% 
  print()

# Correcting error bars to account for stacked barchart
allDays.meanAbsDispersed$ymin[allDays.meanAbsDispersed$patch == 1] <- with(allDays.meanAbsDispersed,ymin[patch == 1] +
                                                                             ymin[patch == 2])
allDays.meanAbsDispersed$ymax[allDays.meanAbsDispersed$patch == 1] <- with(allDays.meanAbsDispersed,ymax[patch == 1] +
                                                                             ymax[patch == 2])

# Plotting
facetGridNamesCols <- c(`N` = "Patch 2 not fed", 
                        `Y` = "Patch 2 well fed")
facetGridNamesRows <- c(`N` = "Patch 1 not fed", 
                        `Y` = "Patch 1 well fed")
ggplot(allDays.meanAbsDispersed, aes(x=day,y=totalPop,fill=as.factor(patch),ymin=ymin,ymax=ymax, colour=as.factor(patch))) +
  geom_col(position="stack") +
  geom_errorbar(width=0.2) +
  facet_grid(`patch1Fed?`~`patch2Fed?`, labeller = labeller(.rows = facetGridNamesRows,.cols = facetGridNamesCols)) +
  labs(y="Mean population size", x="Time (days of dispersal)") +
  scale_fill_grey(name = "", labels = c("Patch 1", "Patch 2"), start = 0, end = 0.5) +
  scale_colour_grey(name = "", labels = c("Patch 1", "Patch 2"), start = 0, end = 0.5) +
  #scale_y_continuous(expand = c(0, 0), limits = c(0,225)) +
  theme_bw(base_size = 16) +
  theme(legend.position = "bottom")
#ggsave(filename = "Figures/figPopSizeTime.pdf", width = 20, height = 20, units = "cm")

# The age of individuals in patch 2 on the final day
# Manipulating data for plotting
allDays.age <- CDD %>%
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","patch","day")) %>% 
  filter(patch == 2) %>%
  gather(`juveniles`,`adults`,key=age,value=popSize) %>% 
  group_by(.dots=c("`patch1Fed?`","`patch2Fed?`","patch","day", "age")) %>% 
  summarize(total = mean(popSize), 
            n = n(), 
            sd = sd(popSize),
            se = sd/sqrt(n),
            ymin = total-se, ymax = total+se) %>% 
  print()

# Plotting
ggplot(allDays.age,aes(y=total,x=day,fill=age,ymin=ymin,ymax=ymax,colour=age)) +
  geom_col(position = position_stack(reverse = T)) +
  geom_errorbar(width=0.2) +
  facet_grid(`patch1Fed?`~`patch2Fed?`, labeller = labeller(.rows = facetGridNamesRows,.cols = facetGridNamesCols)) +
  labs(y="Mean patch 2 population size", x="Time (days of dispersal)") +
  scale_fill_grey(name = "", labels = c("Adults", "Juveniles"), start = 0, end = 0.5) +
  scale_colour_grey(name = "", labels = c("Adults", "Juveniles"), start = 0, end = 0.5) +
  #scale_y_continuous(expand = c(0, 0), limits = c(0,225)) +
  theme_bw(base_size = 16) +
  theme(legend.position = "bottom")
#ggsave(filename = "Figures/figPopAgeTime.pdf", width = 20, height = 20, units = "cm")

# The correlation between density and proportion of individuals dispersing
# Function for generating r^2 and applying to figure
stat_smooth_func <- function(mapping = NULL, data = NULL,
                             geom = "smooth", position = "identity",
                             ...,
                             method = "auto",
                             formula = y ~ x,
                             se = TRUE,
                             n = 80,
                             span = 0.75,
                             fullrange = FALSE,
                             level = 0.95,
                             method.args = list(),
                             na.rm = FALSE,
                             show.legend = NA,
                             inherit.aes = TRUE,
                             xpos = NULL,
                             ypos = NULL) {
  layer(
    data = data,
    mapping = mapping,
    stat = StatSmoothFunc,
    geom = geom,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      method = method,
      formula = formula,
      se = se,
      n = n,
      fullrange = fullrange,
      level = level,
      na.rm = na.rm,
      method.args = method.args,
      span = span,
      xpos = xpos,
      ypos = ypos,
      ...
    )
  )
}


StatSmoothFunc <- ggproto("StatSmooth", Stat,
                          
                          setup_params = function(data, params) {
                            # Figure out what type of smoothing to do: loess for small datasets,
                            # gam with a cubic regression basis for large data
                            # This is based on the size of the _largest_ group.
                            if (identical(params$method, "auto")) {
                              max_group <- max(table(data$group))
                              
                              if (max_group < 1000) {
                                params$method <- "loess"
                              } else {
                                params$method <- "gam"
                                params$formula <- y ~ s(x, bs = "cs")
                              }
                            }
                            if (identical(params$method, "gam")) {
                              params$method <- mgcv::gam
                            }
                            
                            params
                          },
                          
                          compute_group = function(data, scales, method = "auto", formula = y~x,
                                                   se = TRUE, n = 80, span = 0.75, fullrange = FALSE,
                                                   xseq = NULL, level = 0.95, method.args = list(),
                                                   na.rm = FALSE, xpos=NULL, ypos=NULL) {
                            if (length(unique(data$x)) < 2) {
                              # Not enough data to perform fit
                              return(data.frame())
                            }
                            
                            if (is.null(data$weight)) data$weight <- 1
                            
                            if (is.null(xseq)) {
                              if (is.integer(data$x)) {
                                if (fullrange) {
                                  xseq <- scales$x$dimension()
                                } else {
                                  xseq <- sort(unique(data$x))
                                }
                              } else {
                                if (fullrange) {
                                  range <- scales$x$dimension()
                                } else {
                                  range <- range(data$x, na.rm = TRUE)
                                }
                                xseq <- seq(range[1], range[2], length.out = n)
                              }
                            }
                            # Special case span because it's the most commonly used model argument
                            if (identical(method, "loess")) {
                              method.args$span <- span
                            }
                            
                            if (is.character(method)) method <- match.fun(method)
                            
                            base.args <- list(quote(formula), data = quote(data), weights = quote(weight))
                            model <- do.call(method, c(base.args, method.args))
                            
                            m = model
                            eq <- substitute(~~italic(r)^2~"="~r2, 
                                             list(a = format(coef(m)[1], digits = 3), 
                                                  b = format(coef(m)[2], digits = 3), 
                                                  r2 = format(summary(m)$r.squared, digits = 3)))
                            func_string = as.character(as.expression(eq))
                            
                            if(is.null(xpos)) xpos = min(data$x)*0.9
                            if(is.null(ypos)) ypos = max(data$y)*0.9
                            data.frame(x=xpos, y=ypos, label=func_string)
                            
                          },
                          
                          required_aes = c("x", "y")
)

# Plotting
facetGridNamesCols <- c(`N` = "Patch 2 not fed", 
                        `Y` = "Patch 2 well fed")
facetGridNamesRows <- c(`N` = "Patch 1 not fed", 
                        `Y` = "Patch 1 well fed")
ggplot(day4.meanPropDispersed,aes(x=n,y=propDispersed)) +
  geom_smooth(method="lm",se=F) +
  stat_smooth_func(geom="text",method="lm",vjust=-0.25,hjust=-0.2,parse=TRUE) +
  geom_point() +
  facet_grid(`patch1Fed?`~`patch2Fed?`, labeller = labeller(.rows = facetGridNamesRows,.cols = facetGridNamesCols)) +
  labs(y = c(expression("Proportion of"~italic("D. carinata")~"dispersing into patch 2")), x="Population density") +
  theme_bw(base_size = 16) +
  theme(legend.position = "bottom")
#ggsave(filename = "Figures/figPopDensProp.pdf", width = 20, height = 20, units = "cm")

# Figure that ignores patch 2 treatment
ggplot(day4.meanPropDispersed,aes(x=n,y=propDispersed)) +
  geom_smooth(method="lm",se=F) +
  stat_smooth_func(geom="text",method="lm",vjust=-0.25,hjust=-0.2,parse=TRUE) +
  geom_point() +
  facet_grid(.~`patch1Fed?`, labeller = labeller(.cols = facetGridNamesRows)) +
  labs(y = c(expression("Proportion of"~italic("D. carinata")~"dispersing into patch 2")), x="Population density") +
  theme_bw(base_size = 16) +
  theme(legend.position = "bottom")
#ggsave(filename = "Figures/figPopDensPropAlt.pdf", width = 20, height = 20, units = "cm")

# Figure that combines all data onto one panel
ggplot(day4.meanPropDispersed,aes(x=n,y=propDispersed,linetype=`patch1Fed?`,shape=`patch1Fed?`)) +
  geom_smooth(method="lm",se=F) +
  stat_smooth_func(geom="text",method="lm",vjust=2,hjust=-3,parse=TRUE) +
  geom_point() +
  #facet_grid(.~`patch1Fed?`, labeller = labeller(.cols = facetGridNamesRows)) +
  labs(y = c(expression("Proportion of"~italic("D. carinata")~"dispersing into patch 2")), x="Population density") +
  scale_linetype_manual(name = " ", labels = c("Patch 1 not fed","Patch 1 fed"),values = c(1,2)) +
  scale_shape_manual(name = " ", labels = c("Patch 1 not fed","Patch 1 fed"),values = c(16,17)) +
  theme_bw(base_size = 16) +
  theme(legend.title=element_blank(),legend.text.align = 0)
#ggsave(filename = "Figures/figPopDensPropAlt.pdf", width = 20, height = 20, units = "cm")
